@@ -34,6 +34,7 @@
 #include <language/duchain/dumpchain.h>
 
 #include "typeutils.h"
+#include "templatedeclaration.h"
 
 using namespace KDevelop;
 using namespace Cpp;
@@ -601,3 +602,217 @@ void TestDUChain::testLambdaCapture()
   QVERIFY(funType->returnType().cast<IntegralType>());
   QCOMPARE(funType->returnType().cast<IntegralType>()->dataType(), (uint) IntegralType::TypeVoid);
 }
+
+void TestDUChain::testTemplateSpecializeArray()
+{
+  // see also: https://bugs.kde.org/show_bug.cgi?id=294306
+  const QByteArray code(
+    "template <typename T>\n"
+    "class test\n"
+    "{\n"
+    "public:\n"
+    "    void foo() {}\n"
+    "};\n"
+    "template<typename T>\n"
+    "class test<T[]>\n"
+    "{\n"
+    "};\n"
+    "int main() {\n"
+    "    test<int> t1;\n"
+    "    // should work:\n"
+    "    t1.foo();\n"
+    "    test<int[]> t2;\n"
+    "    // should not work:\n"
+    "    t2.foo();\n"
+    "\n"
+    "    return 0;\n"
+    "}\n"
+  );
+  LockedTopDUContext top = parse(code, DumpAll);
+  QVERIFY(top);
+  DUChainReadLocker lock;
+  QVERIFY(top->problems().isEmpty());
+
+  QCOMPARE(top->localDeclarations().size(), 3);
+
+  Declaration* tplDecRaw = top->localDeclarations().at(0);
+  TemplateDeclaration* tplDec = dynamic_cast<TemplateDeclaration*>(tplDecRaw);
+  QVERIFY(tplDec);
+  QCOMPARE(tplDec->specializationsSize(), 1u);
+  QCOMPARE(tplDec->instantiations().size(), 2);
+
+  Declaration* specRaw = top->localDeclarations().at(1);
+  TemplateDeclaration* spec = dynamic_cast<TemplateDeclaration*>(specRaw);
+  QVERIFY(spec);
+  QVERIFY(spec->specializedFrom().isValid());
+  QVERIFY(spec->specializedWith().isValid());
+  InstantiationInformation info = spec->specializedWith().information();
+  QCOMPARE(info.templateParametersSize(), 1u);
+  QVERIFY(info.templateParameters()[0].isValid());
+  AbstractType::Ptr specParam = info.templateParameters()[0].abstractType();
+  QVERIFY(specParam);
+  QVERIFY(specParam.cast<ArrayType>());
+  QCOMPARE(specParam->toString(), QString("T[]"));
+
+  QCOMPARE(top->childContexts().last()->localDeclarations().size(), 2);
+  QCOMPARE(top->childContexts().last()->localDeclarations().at(0)->abstractType()->toString(),
+           QString("test< int >"));
+  QCOMPARE(top->childContexts().last()->localDeclarations().at(1)->abstractType()->toString(),
+           QString("test< int[] >"));
+  // should work
+  QVERIFY(top->childContexts().last()->findUseAt(CursorInRevision(13, 9)) != -1);
+  // should not work
+  QVERIFY(top->childContexts().last()->findUseAt(CursorInRevision(16, 9)) == -1);
+}
+
+void TestDUChain::testTemplateSpecializeRValue()
+{
+  const QByteArray code(
+    "template <typename T>\n"
+    "class test\n"
+    "{\n"
+    "public:\n"
+    "    void foo() {}\n"
+    "};\n"
+    "template<typename T>\n"
+    "class test<T&&>\n"
+    "{\n"
+    "};\n"
+    "int main() {\n"
+    "    test<int> t1;\n"
+    "    // should work:\n"
+    "    t1.foo();\n"
+    "    test<int&&> t2;\n"
+    "    // should not work:\n"
+    "    t2.foo();\n"
+    "\n"
+    "    return 0;\n"
+    "}\n"
+  );
+  LockedTopDUContext top = parse(code, DumpAll);
+  QVERIFY(top);
+  DUChainReadLocker lock;
+  QVERIFY(top->problems().isEmpty());
+
+  QCOMPARE(top->localDeclarations().size(), 3);
+
+  Declaration* tplDecRaw = top->localDeclarations().at(0);
+  TemplateDeclaration* tplDec = dynamic_cast<TemplateDeclaration*>(tplDecRaw);
+  QVERIFY(tplDec);
+  QCOMPARE(tplDec->specializationsSize(), 1u);
+  QCOMPARE(tplDec->instantiations().size(), 2);
+
+  Declaration* specRaw = top->localDeclarations().at(1);
+  TemplateDeclaration* spec = dynamic_cast<TemplateDeclaration*>(specRaw);
+  QVERIFY(spec);
+  QVERIFY(spec->specializedFrom().isValid());
+  QVERIFY(spec->specializedWith().isValid());
+  InstantiationInformation info = spec->specializedWith().information();
+  QCOMPARE(info.templateParametersSize(), 1u);
+  QVERIFY(info.templateParameters()[0].isValid());
+  AbstractType::Ptr specParam = info.templateParameters()[0].abstractType();
+  QVERIFY(specParam);
+  QVERIFY(specParam.cast<ReferenceType>());
+  QVERIFY(specParam.cast<ReferenceType>()->isRValue());
+  QCOMPARE(specParam->toString(), QString("T&&"));
+
+  QCOMPARE(top->childContexts().last()->localDeclarations().size(), 2);
+  QCOMPARE(top->childContexts().last()->localDeclarations().at(0)->abstractType()->toString(),
+           QString("test< int >"));
+  QCOMPARE(top->childContexts().last()->localDeclarations().at(1)->abstractType()->toString(),
+           QString("test< int&& >"));
+
+  // should work
+  QVERIFY(top->childContexts().last()->findUseAt(CursorInRevision(13, 9)) != -1);
+  // should not work
+  QVERIFY(top->childContexts().last()->findUseAt(CursorInRevision(16, 9)) == -1);
+}
+
+void TestDUChain::testAliasDeclaration()
+{
+  const QByteArray code(
+    "struct foo { static void asdf(); };\n"
+    "using bar = foo;\n"
+    "void t() { bar::asdf(); }"
+  );
+  LockedTopDUContext top = parse(code, DumpAll);
+  QVERIFY(top);
+  DUChainReadLocker lock;
+  QVERIFY(top->problems().isEmpty());
+
+  QCOMPARE(top->localDeclarations().count(), 3);
+
+  // foo
+  Declaration* dec = top->localDeclarations().at(0);
+  QCOMPARE(dec->kind(), Declaration::Type);
+  QCOMPARE(dec->uses().count(), 1);
+  QCOMPARE(dec->uses().begin()->count(), 1);
+  QCOMPARE(dec->uses().begin()->first().start.line, 1);
+
+  // bar
+  dec = top->localDeclarations().at(1);
+  QVERIFY(dec->isTypeAlias());
+  QCOMPARE(dec->uses().count(), 1);
+  QCOMPARE(dec->uses().begin()->count(), 1);
+  QCOMPARE(dec->uses().begin()->first().start.line, 2);
+
+  // foo::asdf
+  dec = top->childContexts().first()->localDeclarations().at(0);
+  QCOMPARE(dec->uses().count(), 1);
+  QCOMPARE(dec->uses().begin()->count(), 1);
+  QCOMPARE(dec->uses().begin()->first().start.line, 2);
+}
+
+void TestDUChain::testAuto()
+{
+  const QByteArray code(
+    "char i = 1;\n"
+    "auto a1 = i;\n"
+    "auto& a2 = i;\n"
+    "auto&& a3 = i;\n"
+    "const auto a4 = i;\n"
+    "const auto& a5 = i;\n"
+    "const auto&& a6 = i;\n"
+  );
+  LockedTopDUContext top = parse(code, DumpAll);
+  QVERIFY(top);
+  DUChainReadLocker lock;
+  QVERIFY(top->problems().isEmpty());
+
+  QCOMPARE(top->localDeclarations().count(), 7);
+
+  Declaration* dec = top->localDeclarations().at(1);
+  QVERIFY(dec->type<IntegralType>());
+  QCOMPARE(dec->type<IntegralType>()->dataType(), (uint) IntegralType::TypeChar);
+
+  dec = top->localDeclarations().at(2);
+  QVERIFY(dec->type<ReferenceType>());
+  QVERIFY(!dec->type<ReferenceType>()->isRValue());
+  QVERIFY(dec->type<ReferenceType>()->baseType().cast<IntegralType>());
+  QCOMPARE(dec->type<ReferenceType>()->baseType().cast<IntegralType>()->dataType(), (uint) IntegralType::TypeChar);
+
+  dec = top->localDeclarations().at(3);
+  QVERIFY(dec->type<ReferenceType>());
+  QVERIFY(dec->type<ReferenceType>()->isRValue());
+  QVERIFY(dec->type<ReferenceType>()->baseType().cast<IntegralType>());
+  QCOMPARE(dec->type<ReferenceType>()->baseType().cast<IntegralType>()->dataType(), (uint) IntegralType::TypeChar);
+
+  dec = top->localDeclarations().at(4);
+  QVERIFY(dec->type<IntegralType>());
+  QVERIFY(dec->abstractType()->modifiers() & AbstractType::ConstModifier);
+  QCOMPARE(dec->type<IntegralType>()->dataType(), (uint) IntegralType::TypeChar);
+
+  dec = top->localDeclarations().at(5);
+  QVERIFY(dec->type<ReferenceType>());
+  QVERIFY(dec->type<ReferenceType>()->baseType()->modifiers() & AbstractType::ConstModifier);
+  QVERIFY(!dec->type<ReferenceType>()->isRValue());
+  QVERIFY(dec->type<ReferenceType>()->baseType().cast<IntegralType>());
+  QCOMPARE(dec->type<ReferenceType>()->baseType().cast<IntegralType>()->dataType(), (uint) IntegralType::TypeChar);
+
+  dec = top->localDeclarations().at(6);
+  QVERIFY(dec->type<ReferenceType>());
+  QVERIFY(dec->type<ReferenceType>()->baseType()->modifiers() & AbstractType::ConstModifier);
+  QVERIFY(dec->type<ReferenceType>()->isRValue());
+  QVERIFY(dec->type<ReferenceType>()->baseType().cast<IntegralType>());
+  QCOMPARE(dec->type<ReferenceType>()->baseType().cast<IntegralType>()->dataType(), (uint) IntegralType::TypeChar);
+}
